@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2020 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -203,14 +203,9 @@ foreach ($data['data']['problems'] as $eventid => $problem) {
 
 	$problem_link = [
 		(new CLinkAction($problem['name']))
-			->setHint(
-				make_popup_eventlist(['comments' => $problem['comments'], 'url' => $problem['url'],
-					'triggerid' => $trigger['triggerid']], $eventid, $allowed, $show_timeline, $data['fields']['show_tags'],
-					$data['fields']['tags'], $data['fields']['tag_name_format'], $data['fields']['tag_priority']
-				)
-			)
+			->setMenuPopup(CMenuPopupHelper::getTrigger($trigger['triggerid'], $problem['eventid']))
 			->setAttribute('aria-label', _xs('%1$s, Severity, %2$s', 'screen reader',
-				$problem['name'], getSeverityName($problem['severity'])
+				$problem['name'], CSeverityHelper::getName((int) $problem['severity'])
 			))
 	];
 
@@ -220,7 +215,7 @@ foreach ($data['data']['problems'] as $eventid => $problem) {
 
 	$description = (new CCol($problem_link));
 
-	$description_style = getSeverityStyle($problem['severity']);
+	$description_style = CSeverityHelper::getStyle((int) $problem['severity']);
 
 	if ($value == TRIGGER_VALUE_TRUE) {
 		$description->addClass($description_style);
@@ -281,12 +276,15 @@ foreach ($data['data']['problems'] as $eventid => $problem) {
 		$triggers_hosts[$trigger['triggerid']],
 		$description,
 		($show_opdata == OPERATIONAL_DATA_SHOW_SEPARATELY ) ? $opdata : null,
-		(new CCol(zbx_date2age($problem['clock'], ($problem['r_eventid'] != 0) ? $problem['r_clock'] : 0)))
-			->addClass(ZBX_STYLE_NOWRAP),
+		(new CCol(
+			(new CLinkAction(zbx_date2age($problem['clock'], ($problem['r_eventid'] != 0) ? $problem['r_clock'] : 0)))
+				->setAjaxHint(CHintBoxHelper::getEventList($trigger['triggerid'], $eventid, $show_timeline,
+					$data['fields']['show_tags'], $data['fields']['tags'], $data['fields']['tag_name_format'],
+					$data['fields']['tag_priority']
+				))
+		))->addClass(ZBX_STYLE_NOWRAP),
 		$problem_update_link,
-		makeEventActionsIcons($problem['eventid'], $data['data']['actions'], $data['data']['mediatypes'],
-			$data['data']['users']
-		),
+		makeEventActionsIcons($problem['eventid'], $data['data']['actions'], $data['data']['users']),
 		$data['fields']['show_tags'] ? $data['data']['tags'][$problem['eventid']] : null
 	]));
 }
@@ -300,22 +298,10 @@ if ($data['info'] !== '') {
 }
 
 $output = [
-	'aria_label' => _xs('%1$s widget', 'screen reader', $data['name']).', '.$data['info'],
-	'header' => $data['name'],
+	'name' => $data['name'],
 	'body' => $table->toString()
 ];
 
-if ($data['initial_load']) {
-	$output['script_inline'] =
-		'if (typeof refreshProblemsWidget !== typeof(Function)) {'.
-			'function refreshProblemsWidget(event, response, overlay) {'.
-				'refreshWidgetOnAcknowledgeCreate("problems", response, overlay);'.
-			'}'.
-
-			'$.subscribe("acknowledge.create", refreshProblemsWidget);'.
-		'}';
-}
-
 if (($messages = getMessages()) !== null) {
 	$output['messages'] = $messages->toString();
 }
