@@ -23,10 +23,6 @@
  * @var CView $this
  */
 
-// Visibility box javascript is already added. It should not be added in popup response.
-define('CVISIBILITYBOX_JAVASCRIPT_INSERTED', 1);
-define('IS_TEXTAREA_MAXLENGTH_JS_INSERTED', 1);
-
 // Create form.
 $form = (new CForm())
 	->setId('massupdate-form')
@@ -53,10 +49,7 @@ $trigger_form_list = (new CFormList('trigger-form-list'))
 			->setLabel(_('Severity'))
 			->setAttribute('autofocus', 'autofocus'),
 		(new CDiv(
-			new CSeverity([
-				'name' => 'priority',
-				'value' => 0
-			])
+			new CSeverity('priority', 0)
 		))->setId('priority-div')
 	)
 	->addRow(
@@ -103,8 +96,8 @@ $dependencies_table = (new CTable())
 $bttn_prototype = '';
 if ($data['prototype']) {
 	$bttn_prototype = (new CButton('add_dep_trigger_prototype', _('Add prototype')))
-	->onClick('return PopUp("popup.generic",'.
-		json_encode([
+	->onClick(
+		'return PopUp("popup.generic", '.json_encode([
 			'srctbl' => 'trigger_prototypes',
 			'srcfld1' => 'triggerid',
 			'dstfrm' => 'massupdate',
@@ -114,7 +107,7 @@ if ($data['prototype']) {
 			'multiselect' => '1',
 			'objname' => 'triggers',
 			'parent_discoveryid' => $data['parent_discoveryid']
-		]).', null, this);'
+		]).', {dialogue_class: "modal-popup-generic"});'
 	)
 	->addClass(ZBX_STYLE_BTN_LINK);
 }
@@ -126,8 +119,8 @@ $dependencies_form_list->addRow(
 		$dependencies_table,
 		new CHorList([
 			(new CButton('btn1', _('Add')))
-				->onClick('return PopUp("popup.generic",'.
-					json_encode([
+				->onClick(
+					'return PopUp("popup.generic", '.json_encode([
 						'srctbl' => 'triggers',
 						'srcfld1' => 'triggerid',
 						'dstfrm' => 'massupdate',
@@ -139,7 +132,7 @@ $dependencies_form_list->addRow(
 						'with_triggers' => '1',
 						'normal_only' => '1',
 						'noempty' => '1'
-					]).', null, this);'
+					]).', {dialogue_class: "modal-popup-generic"});'
 				)
 				->addClass(ZBX_STYLE_BTN_LINK),
 			$bttn_prototype
