@@ -655,7 +655,7 @@ class CExpressionParser extends CParser {
 			int $depth): bool {
 		$p = $pos;
 
-		if (!preg_match('/^([a-z0-9]+)\(/', substr($source, $p), $matches)) {
+		if (!preg_match('/^([a-z0-9_]+)\(/', substr($source, $p), $matches)) {
 			return false;
 		}
 
@@ -758,7 +758,11 @@ class CExpressionParser extends CParser {
 	 * @return bool returns true if parsed successfully, false otherwise
 	 */
 	private static function parseNumber(string $source, int &$pos, array &$tokens): bool {
-		$number_parser = new CNumberParser(['with_minus' => false, 'with_suffix' => true]);
+		$number_parser = new CNumberParser([
+			'with_minus' => false,
+			'with_size_suffix' => true,
+			'with_time_suffix' => true
+		]);
 
 		if ($number_parser->parse($source, $pos) == CParser::PARSE_FAIL) {
 			return false;
@@ -830,7 +834,7 @@ class CExpressionParser extends CParser {
 	 */
 	public static function quoteString(string $value, bool $allow_macros = true, bool $force = false): string {
 		if (!$force) {
-			$number_parser = new CNumberParser(['with_suffix' => true]);
+			$number_parser = new CNumberParser(['with_size_suffix' => true, 'with_time_suffix' => true]);
 
 			if ($number_parser->parse($value) == CParser::PARSE_SUCCESS) {
 				return $value;
