@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2020 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -33,19 +33,16 @@ function eventSource($source = null) {
 	$sources = [
 		EVENT_SOURCE_TRIGGERS => _('trigger'),
 		EVENT_SOURCE_DISCOVERY => _('discovery'),
-		EVENT_SOURCE_AUTO_REGISTRATION => _('auto registration'),
-		EVENT_SOURCE_INTERNAL => _x('internal', 'event source')
+		EVENT_SOURCE_AUTOREGISTRATION => _('autoregistration'),
+		EVENT_SOURCE_INTERNAL => _x('internal', 'event source'),
+		EVENT_SOURCE_SERVICE => _('service')
 	];
 
 	if ($source === null) {
 		return $sources;
 	}
-	elseif (isset($sources[$source])) {
-		return $sources[$source];
-	}
-	else {
-		return _('Unknown');
-	}
+
+	return array_key_exists($source, $sources) ?  $sources[$source] : _('Unknown');
 }
 
 /**
@@ -63,9 +60,10 @@ function eventObject($object = null) {
 		EVENT_OBJECT_TRIGGER => _('trigger'),
 		EVENT_OBJECT_DHOST => _('discovered host'),
 		EVENT_OBJECT_DSERVICE => _('discovered service'),
-		EVENT_OBJECT_AUTOREGHOST => _('auto-registered host'),
+		EVENT_OBJECT_AUTOREGHOST => _('autoregistered host'),
 		EVENT_OBJECT_ITEM => _('item'),
-		EVENT_OBJECT_LLDRULE => _('low-level discovery rule')
+		EVENT_OBJECT_LLDRULE => _('low-level discovery rule'),
+		EVENT_OBJECT_SERVICE => _('service')
 	];
 
 	if ($object === null) {
@@ -89,10 +87,11 @@ function eventSourceObjects() {
 		['source' => EVENT_SOURCE_TRIGGERS, 'object' => EVENT_OBJECT_TRIGGER],
 		['source' => EVENT_SOURCE_DISCOVERY, 'object' => EVENT_OBJECT_DHOST],
 		['source' => EVENT_SOURCE_DISCOVERY, 'object' => EVENT_OBJECT_DSERVICE],
-		['source' => EVENT_SOURCE_AUTO_REGISTRATION, 'object' => EVENT_OBJECT_AUTOREGHOST],
+		['source' => EVENT_SOURCE_AUTOREGISTRATION, 'object' => EVENT_OBJECT_AUTOREGHOST],
 		['source' => EVENT_SOURCE_INTERNAL, 'object' => EVENT_OBJECT_TRIGGER],
 		['source' => EVENT_SOURCE_INTERNAL, 'object' => EVENT_OBJECT_ITEM],
-		['source' => EVENT_SOURCE_INTERNAL, 'object' => EVENT_OBJECT_LLDRULE]
+		['source' => EVENT_SOURCE_INTERNAL, 'object' => EVENT_OBJECT_LLDRULE],
+		['source' => EVENT_SOURCE_SERVICE, 'object' => EVENT_OBJECT_SERVICE]
 	];
 }
 
@@ -104,12 +103,11 @@ function get_events_unacknowledged($db_element, $value_trigger = null, $value_ev
 		return 0;
 	}
 
-	$config = select_config();
 	$options = [
 		'output' => ['triggerid'],
 		'monitored' => 1,
 		'skipDependent' => 1,
-		'limit' => $config['search_limit'] + 1
+		'limit' => CSettingsHelper::get(CSettingsHelper::SEARCH_LIMIT) + 1
 	];
 	if (!is_null($value_trigger)) {
 		$options['filter'] = ['value' => $value_trigger];
@@ -139,32 +137,48 @@ function get_events_unacknowledged($db_element, $value_trigger = null, $value_ev
 
 /**
  *
- * @param array  $event                   An array of event data.
- * @param string $event['eventid']        Event ID.
- * @param string $event['objectid']       Object ID.
- * @param string $event['correlationid']  OK Event correlation ID.
- * @param string $event['userid']         User ID who generated the OK event.
- * @param string $event['name']           Event name.
- * @param string $event['acknowledged']   State of acknowledgement.
- * @param CCOl   $event['opdata']         Operational data with expanded macros.
- * @param string $event['comments']       Trigger description with expanded macros.
- * @param string $backurl                 A link back after acknowledgement has been clicked.
+ * @param array  $event                              An array of event data.
+ * @param string $event['eventid']                   Event ID.
+ * @param string $event['r_eventid']                 OK event ID.
+ * @param string $event['objectid']                  Object ID.
+ * @param string $event['correlationid']             OK Event correlation ID.
+ * @param string $event['userid']                    User ID who generated the OK event.
+ * @param string $event['name']                      Event name.
+ * @param string $event['acknowledged']              State of acknowledgement.
+ * @param array  $event['acknowledges']              List of problem updates.
+ * @param string $event['acknowledges'][]['action']  Action performed in update.
+ * @param CCOl   $event['opdata']                    Operational data with expanded macros.
+ * @param string $event['comments']                  Trigger description with expanded macros.
+ * @param array  $allowed                            An array of user role rules.
+ * @param bool   $allowed['ui_correlation']          Whether user is allowed to visit event correlation page.
+ * @param bool   $allowed['add_comments']            Whether user is allowed to add problems comments.
+ * @param bool   $allowed['change_severity']         Whether user is allowed to change problems severity.
+ * @param bool   $allowed['acknowledge']             Whether user is allowed to acknowledge problems.
+ * @param bool   $allowed['close']                   Whether user is allowed to close problems.
  *
  * @return CTableInfo
  */
-function make_event_details($event, $backurl) {
-	$event_update_url = (new CUrl('zabbix.php'))
-		->setArgument('action', 'acknowledge.edit')
-		->setArgument('eventids', [$event['eventid']])
-		->setArgument('backurl', $backurl)
-		->getUrl();
+function make_event_details(array $event, array $allowed) {
+	$can_be_closed = $allowed['close'];
 
-	$config = select_config();
+	if ($event['r_eventid'] != 0) {
+		$can_be_closed = false;
+	}
+	else {
+		foreach ($event['acknowledges'] as $acknowledge) {
+			if (($acknowledge['action'] & ZBX_PROBLEM_UPDATE_CLOSE) == ZBX_PROBLEM_UPDATE_CLOSE) {
+				$can_be_closed = false;
+				break;
+			}
+		}
+	}
+
+	$is_acknowledged = ($event['acknowledged'] == EVENT_ACKNOWLEDGED);
 
 	$table = (new CTableInfo())
 		->addRow([
 			_('Event'),
-			$event['name']
+			(new CCol($event['name']))->addClass(ZBX_STYLE_WORDWRAP)
 		])
 		->addRow([
 			_('Operational data'),
@@ -172,7 +186,7 @@ function make_event_details($event, $backurl) {
 		])
 		->addRow([
 			_('Severity'),
-			getSeverityCell($event['severity'], $config)
+			CSeverityHelper::makeSeverityCell((int) $event['severity'])
 		])
 		->addRow([
 			_('Time'),
@@ -180,9 +194,14 @@ function make_event_details($event, $backurl) {
 		])
 		->addRow([
 			_('Acknowledged'),
-			(new CLink($event['acknowledged'] == EVENT_ACKNOWLEDGED ? _('Yes') : _('No'), $event_update_url))
-				->addClass($event['acknowledged'] == EVENT_ACKNOWLEDGED ? ZBX_STYLE_GREEN : ZBX_STYLE_RED)
-				->addClass(ZBX_STYLE_LINK_ALT)
+			($allowed['add_comments'] || $allowed['change_severity'] || $allowed['acknowledge'] || $can_be_closed)
+				? (new CLink($is_acknowledged ? _('Yes') : _('No')))
+					->addClass($is_acknowledged ? ZBX_STYLE_GREEN : ZBX_STYLE_RED)
+					->addClass(ZBX_STYLE_LINK_ALT)
+					->onClick('acknowledgePopUp('.json_encode(['eventids' => [$event['eventid']]]).', this);')
+				: (new CSpan($is_acknowledged ? _('Yes') : _('No')))->addClass(
+					$is_acknowledged ? ZBX_STYLE_GREEN : ZBX_STYLE_RED
+				)
 		]);
 
 	if ($event['r_eventid'] != 0) {
@@ -193,9 +212,10 @@ function make_event_details($event, $backurl) {
 			]);
 
 			if ($correlations) {
-				if (CWebUser::getType() == USER_TYPE_SUPER_ADMIN) {
+				if ($allowed['ui_correlation']) {
 					$correlation_name = (new CLink($correlations[0]['name'],
-						(new CUrl('correlation.php'))
+						(new CUrl('zabbix.php'))
+							->setArgument('action', 'correlation.edit')
 							->setArgument('correlationid', $correlations[0]['correlationid'])
 							->getUrl()
 					))->addClass(ZBX_STYLE_LINK_ALT);
@@ -213,14 +233,14 @@ function make_event_details($event, $backurl) {
 		elseif ($event['userid'] != 0) {
 			if ($event['userid'] == CWebUser::$data['userid']) {
 				$table->addRow([_('Resolved by'), getUserFullname([
-					'alias' => CWebUser::$data['alias'],
+					'username' => CWebUser::$data['username'],
 					'name' => CWebUser::$data['name'],
 					'surname' => CWebUser::$data['surname']
 				])]);
 			}
 			else {
 				$user = API::User()->get([
-					'output' => ['alias', 'name', 'surname'],
+					'output' => ['username', 'name', 'surname'],
 					'userids' => [$event['userid']]
 				]);
 
@@ -246,9 +266,20 @@ function make_event_details($event, $backurl) {
 	return $table;
 }
 
-function make_small_eventlist($startEvent, $backurl) {
-	$config = select_config();
-
+/**
+ *
+ * @param array  $startEvent                  An array of event data.
+ * @param string $startEvent['eventid']       Event ID.
+ * @param string $startEvent['objectid']      Object ID.
+ * @param array  $allowed                     An array of user role rules.
+ * @param bool   $allowed['add_comments']     Whether user is allowed to add problems comments.
+ * @param bool   $allowed['change_severity']  Whether user is allowed to change problems severity.
+ * @param bool   $allowed['acknowledge']      Whether user is allowed to acknowledge problems.
+ * @param bool   $allowed['close']            Whether user is allowed to close problems.
+ *
+ * @return CTableInfo
+ */
+function make_small_eventlist(array $startEvent, array $allowed) {
 	$table = (new CTableInfo())
 		->setHeader([
 			_('Time'),
@@ -310,37 +341,33 @@ function make_small_eventlist($startEvent, $backurl) {
 		])
 		: [];
 
-	$severity_config = [
-		'severity_name_0' => $config['severity_name_0'],
-		'severity_name_1' => $config['severity_name_1'],
-		'severity_name_2' => $config['severity_name_2'],
-		'severity_name_3' => $config['severity_name_3'],
-		'severity_name_4' => $config['severity_name_4'],
-		'severity_name_5' => $config['severity_name_5']
-	];
-	$actions = getEventsActionsIconsData($events, $triggers, $r_events);
+	$actions = getEventsActionsIconsData($events, $triggers);
 	$users = API::User()->get([
-		'output' => ['alias', 'name', 'surname'],
+		'output' => ['username', 'name', 'surname'],
 		'userids' => array_keys($actions['userids']),
 		'preservekeys' => true
 	]);
-	$mediatypes = API::Mediatype()->get([
-		'output' => ['name', 'maxattempts'],
-		'mediatypeids' => array_keys($actions['mediatypeids']),
-		'preservekeys' => true
-	]);
 
 	foreach ($events as $event) {
 		$duration = ($event['r_eventid'] != 0)
 			? zbx_date2age($event['clock'], $event['r_clock'])
 			: zbx_date2age($event['clock']);
 
-		if ($event['r_eventid'] == 0) {
+		$can_be_closed = $allowed['close'];
+
+		if ($event['r_eventid'] != 0) {
+			$value = TRIGGER_VALUE_FALSE;
+			$value_str = _('RESOLVED');
+			$value_clock = $event['r_clock'];
+			$can_be_closed = false;
+		}
+		else {
 			$in_closing = false;
 
 			foreach ($event['acknowledges'] as $acknowledge) {
 				if (($acknowledge['action'] & ZBX_PROBLEM_UPDATE_CLOSE) == ZBX_PROBLEM_UPDATE_CLOSE) {
 					$in_closing = true;
+					$can_be_closed = false;
 					break;
 				}
 			}
@@ -349,26 +376,26 @@ function make_small_eventlist($startEvent, $backurl) {
 			$value_str = $in_closing ? _('CLOSING') : _('PROBLEM');
 			$value_clock = $in_closing ? time() : $event['clock'];
 		}
-		else {
-			$value = TRIGGER_VALUE_FALSE;
-			$value_str = _('RESOLVED');
-			$value_clock = $event['r_clock'];
-		}
 
+		$is_acknowledged = ($event['acknowledged'] == EVENT_ACKNOWLEDGED);
 		$cell_status = new CSpan($value_str);
 
 		/*
 		 * Add colors to span depending on configuration and trigger parameters. No blinking added to status,
 		 * since the page is not on autorefresh.
 		 */
-		addTriggerValueStyle($cell_status, $value, $value_clock, $event['acknowledged'] == EVENT_ACKNOWLEDGED);
+		addTriggerValueStyle($cell_status, $value, $value_clock, $is_acknowledged);
 
-		// Create link to Problem update page.
-		$problem_update_url = (new CUrl('zabbix.php'))
-			->setArgument('action', 'acknowledge.edit')
-			->setArgument('eventids', [$event['eventid']])
-			->setArgument('backurl', $backurl)
-			->getUrl();
+		// Create acknowledge link.
+		$problem_update_link = ($allowed['add_comments'] || $allowed['change_severity'] || $allowed['acknowledge']
+				|| $can_be_closed)
+			? (new CLink($is_acknowledged ? _('Yes') : _('No')))
+				->addClass($is_acknowledged ? ZBX_STYLE_GREEN : ZBX_STYLE_RED)
+				->addClass(ZBX_STYLE_LINK_ALT)
+				->onClick('acknowledgePopUp('.json_encode(['eventids' => [$event['eventid']]]).', this);')
+			: (new CSpan($is_acknowledged ? _('Yes') : _('No')))->addClass(
+				$is_acknowledged ? ZBX_STYLE_GREEN : ZBX_STYLE_RED
+			);
 
 		$table->addRow([
 			(new CLink(zbx_date2str(DATE_TIME_FORMAT_SECONDS, $event['clock']),
@@ -382,255 +409,14 @@ function make_small_eventlist($startEvent, $backurl) {
 			$cell_status,
 			zbx_date2age($event['clock']),
 			$duration,
-			(new CLink($event['acknowledged'] == EVENT_ACKNOWLEDGED ? _('Yes') : _('No'), $problem_update_url))
-				->addClass($event['acknowledged'] == EVENT_ACKNOWLEDGED ? ZBX_STYLE_GREEN : ZBX_STYLE_RED)
-				->addClass(ZBX_STYLE_LINK_ALT),
-			makeEventActionsIcons($event['eventid'], $actions['data'], $mediatypes, $users, $severity_config)
+			$problem_update_link,
+			makeEventActionsIcons($event['eventid'], $actions['data'], $users)
 		]);
 	}
 
 	return $table;
 }
 
-/**
- * Create table with trigger description and events.
- *
- * @param array  $trigger                    An array of trigger data.
- * @param string $trigger['triggerid']       Trigger ID to select events.
- * @param string $trigger['comments']        Trigger description.
- * @param string $trigger['url']             Trigger URL.
- * @param string $eventid_till
- * @param string $backurl                    URL to return to.
- * @param bool   $show_timeline              Show time line flag.
- * @param int    $show_tags                  Show tags flag. Possible values:
- *                                             - PROBLEMS_SHOW_TAGS_NONE;
- *                                             - PROBLEMS_SHOW_TAGS_1;
- *                                             - PROBLEMS_SHOW_TAGS_2;
- *                                             - PROBLEMS_SHOW_TAGS_3 (default).
- * @param array  $filter_tags                An array of tag filtering data.
- * @param string $filter_tags[]['tag']       Tag name.
- * @param int    $filter_tags[]['operator']  Tag operator.
- * @param string $filter_tags[]['value']     Tag value.
- * @param int    $tag_name_format            Tag name format. Possible values:
- *                                             - PROBLEMS_TAG_NAME_FULL (default);
- *                                             - PROBLEMS_TAG_NAME_SHORTENED;
- *                                             - PROBLEMS_TAG_NAME_NONE.
- * @param string $tag_priority               A list of comma-separated tag names.
- *
- * @return CDiv
- */
-function make_popup_eventlist($trigger, $eventid_till, $backurl, $show_timeline = true,
-		$show_tags = PROBLEMS_SHOW_TAGS_3, array $filter_tags = [], $tag_name_format = PROBLEMS_TAG_NAME_FULL,
-		$tag_priority = '') {
-	// Show trigger description and URL.
-	$div = new CDiv();
-
-	if ($trigger['comments'] !== '') {
-		$div->addItem(
-			(new CDiv())
-				->addItem(zbx_str2links($trigger['comments']))
-				->addClass(ZBX_STYLE_OVERLAY_DESCR)
-				->addStyle('max-width: 500px')
-		);
-	}
-
-	if ($trigger['url'] !== '') {
-		$trigger_url = CHtmlUrlValidator::validate($trigger['url'], ['allow_user_macro' => false])
-			? $trigger['url']
-			: 'javascript: alert(\''._s('Provided URL "%1$s" is invalid.', zbx_jsvalue($trigger['url'], false, false)).
-				'\');';
-
-		$div->addItem(
-			(new CDiv())
-				->addItem(new CLink(CHTML::encode($trigger['url']), $trigger_url))
-				->addClass(ZBX_STYLE_OVERLAY_DESCR_URL)
-				->addStyle('max-width: 500px')
-		);
-	}
-
-	// indicator of sort field
-	$sort_div = (new CSpan())->addClass(ZBX_STYLE_ARROW_DOWN);
-
-	if ($show_timeline) {
-		$header = [
-			(new CColHeader([_('Time'), $sort_div]))->addClass(ZBX_STYLE_RIGHT),
-			(new CColHeader())->addClass(ZBX_STYLE_TIMELINE_TH),
-			(new CColHeader())->addClass(ZBX_STYLE_TIMELINE_TH)
-		];
-	}
-	else {
-		$header = [[_('Time'), $sort_div]];
-	}
-
-	// Select and show events.
-	$table = (new CTableInfo())
-		->setHeader(array_merge($header, [
-			_('Recovery time'),
-			_('Status'),
-			_('Duration'),
-			_('Ack'),
-			($show_tags != PROBLEMS_SHOW_TAGS_NONE) ? _('Tags') : null
-		]));
-
-	if ($eventid_till != 0) {
-		$problems = API::Event()->get([
-			'output' => ['eventid', 'r_eventid', 'clock', 'ns', 'objectid', 'acknowledged'],
-			'selectTags' => ['tag', 'value'],
-			'select_acknowledges' => ['action'],
-			'source' => EVENT_SOURCE_TRIGGERS,
-			'object' => EVENT_OBJECT_TRIGGER,
-			'eventid_till' => $eventid_till,
-			'objectids' => $trigger['triggerid'],
-			'value' => TRIGGER_VALUE_TRUE,
-			'sortfield' => ['eventid'],
-			'sortorder' => ZBX_SORT_DOWN,
-			'limit' => ZBX_WIDGET_ROWS
-		]);
-
-		CArrayHelper::sort($problems, [
-			['field' => 'clock', 'order' => ZBX_SORT_DOWN],
-			['field' => 'ns', 'order' => ZBX_SORT_DOWN]
-		]);
-
-		$r_eventids = [];
-
-		foreach ($problems as $problem) {
-			$r_eventids[$problem['r_eventid']] = true;
-		}
-		unset($r_eventids[0]);
-
-		$r_events = $r_eventids
-			? API::Event()->get([
-				'output' => ['clock', 'correlationid', 'userid'],
-				'source' => EVENT_SOURCE_TRIGGERS,
-				'object' => EVENT_OBJECT_TRIGGER,
-				'eventids' => array_keys($r_eventids),
-				'preservekeys' => true
-			])
-			: [];
-
-		$today = strtotime('today');
-		$last_clock = 0;
-
-		if ($problems && $show_tags != PROBLEMS_SHOW_TAGS_NONE) {
-			$tags = makeTags($problems, true, 'eventid', $show_tags, $filter_tags, $tag_name_format, $tag_priority);
-		}
-
-		$url_details = (new CUrl('tr_events.php'))
-			->setArgument('triggerid', '')
-			->setArgument('eventid', '');
-
-		foreach ($problems as $problem) {
-			if (array_key_exists($problem['r_eventid'], $r_events)) {
-				$problem['r_clock'] = $r_events[$problem['r_eventid']]['clock'];
-				$problem['correlationid'] = $r_events[$problem['r_eventid']]['correlationid'];
-				$problem['userid'] = $r_events[$problem['r_eventid']]['userid'];
-			}
-			else {
-				$problem['r_clock'] = 0;
-				$problem['correlationid'] = 0;
-				$problem['userid'] = 0;
-			}
-
-			if ($problem['r_eventid'] != 0) {
-				$value = TRIGGER_VALUE_FALSE;
-				$value_str = _('RESOLVED');
-				$value_clock = $problem['r_clock'];
-			}
-			else {
-				$in_closing = false;
-
-				foreach ($problem['acknowledges'] as $acknowledge) {
-					if (($acknowledge['action'] & ZBX_PROBLEM_UPDATE_CLOSE) == ZBX_PROBLEM_UPDATE_CLOSE) {
-						$in_closing = true;
-						break;
-					}
-				}
-
-				$value = $in_closing ? TRIGGER_VALUE_FALSE : TRIGGER_VALUE_TRUE;
-				$value_str = $in_closing ? _('CLOSING') : _('PROBLEM');
-				$value_clock = $in_closing ? time() : $problem['clock'];
-			}
-
-			$url_details
-				->setArgument('triggerid', $problem['objectid'])
-				->setArgument('eventid', $problem['eventid']);
-
-			$cell_clock = ($problem['clock'] >= $today)
-				? zbx_date2str(TIME_FORMAT_SECONDS, $problem['clock'])
-				: zbx_date2str(DATE_TIME_FORMAT_SECONDS, $problem['clock']);
-			$cell_clock = new CCol(new CLink($cell_clock, $url_details));
-			if ($problem['r_eventid'] != 0) {
-				$cell_r_clock = ($problem['r_clock'] >= $today)
-					? zbx_date2str(TIME_FORMAT_SECONDS, $problem['r_clock'])
-					: zbx_date2str(DATE_TIME_FORMAT_SECONDS, $problem['r_clock']);
-				$cell_r_clock = (new CCol(new CLink($cell_r_clock, $url_details)))
-					->addClass(ZBX_STYLE_NOWRAP)
-					->addClass(ZBX_STYLE_RIGHT);
-			}
-			else {
-				$cell_r_clock = '';
-			}
-
-			$cell_status = new CSpan($value_str);
-
-			// add colors and blinking to span depending on configuration and trigger parameters
-			addTriggerValueStyle($cell_status, $value, $value_clock, $problem['acknowledged'] == EVENT_ACKNOWLEDGED);
-
-			if ($show_timeline) {
-				if ($last_clock != 0) {
-					CScreenProblem::addTimelineBreakpoint($table, $last_clock, $problem['clock'], ZBX_SORT_DOWN);
-				}
-				$last_clock = $problem['clock'];
-
-				$row = [
-					$cell_clock->addClass(ZBX_STYLE_TIMELINE_DATE),
-					(new CCol())
-						->addClass(ZBX_STYLE_TIMELINE_AXIS)
-						->addClass(ZBX_STYLE_TIMELINE_DOT),
-					(new CCol())->addClass(ZBX_STYLE_TIMELINE_TD)
-				];
-			}
-			else {
-				$row = [
-					$cell_clock
-						->addClass(ZBX_STYLE_NOWRAP)
-						->addClass(ZBX_STYLE_RIGHT)
-				];
-			}
-
-			// Create acknowledge link.
-			$problem_update_url = (new CUrl('zabbix.php'))
-				->setArgument('action', 'acknowledge.edit')
-				->setArgument('eventids', [$problem['eventid']])
-				->setArgument('backurl', $backurl)
-				->getUrl();
-
-			$acknowledged = $problem['acknowledged'] == EVENT_ACKNOWLEDGED;
-			$problem_update_link = (new CLink($acknowledged ? _('Yes') : _('No'), $problem_update_url))
-				->addClass($acknowledged ? ZBX_STYLE_GREEN : ZBX_STYLE_RED)
-				->addClass(ZBX_STYLE_LINK_ALT);
-
-			$table->addRow(array_merge($row, [
-				$cell_r_clock,
-				$cell_status,
-				(new CCol(
-					($problem['r_eventid'] != 0)
-						? zbx_date2age($problem['clock'], $problem['r_clock'])
-						: zbx_date2age($problem['clock'])
-				))
-					->addClass(ZBX_STYLE_NOWRAP),
-				$problem_update_link,
-				($show_tags != PROBLEMS_SHOW_TAGS_NONE) ? $tags[$problem['eventid']] : null
-			]));
-		}
-	}
-
-	$div->addItem($table);
-
-	return $div;
-}
-
 /**
  * Place filter tags at the beginning of tags array.
  *
@@ -699,24 +485,30 @@ function orderEventTagsByPriority(array $event_tags, array $priorities) {
  * @param bool   $html
  * @param string $key                        Name of tag source ID. Possible values:
  *                                            - 'eventid' - for events and problems (default);
- *                                            - 'hostid' - for hosts;
+ *                                            - 'hostid' - for hosts and host prototypes;
  *                                            - 'templateid' - for templates;
  *                                            - 'triggerid' - for triggers.
  * @param int    $list_tag_count             Maximum number of tags to display.
  * @param array  $filter_tags                An array of tag filtering data.
+ * @param ?array $subfilter_tags             Array of selected sub-filter tags. Null when tags are not clickable.
+ * @param array  $subfilter_tags[<tag>]
+ * @param array  $subfilter_tags[<tag>][<value1>]
+ * @param array  $subfilter_tags[<tag>][<value2>]
+ * @param array  $subfilter_tags[<tag>][<value...>]
  * @param string $filter_tags[]['tag']
  * @param int    $filter_tags[]['operator']
  * @param string $filter_tags[]['value']
  * @param int    $tag_name_format            Tag name format. Possible values:
- *                                            - PROBLEMS_TAG_NAME_FULL (default);
- *                                            - PROBLEMS_TAG_NAME_SHORTENED;
- *                                            - PROBLEMS_TAG_NAME_NONE.
+ *                                            - TAG_NAME_FULL (default);
+ *                                            - TAG_NAME_SHORTENED;
+ *                                            - TAG_NAME_NONE.
  * @param string $tag_priority               A list of comma-separated tag names.
  *
  * @return array
  */
-function makeTags(array $list, $html = true, $key = 'eventid', $list_tag_count = ZBX_TAG_COUNT_DEFAULT,
-		array $filter_tags = [], $tag_name_format = PROBLEMS_TAG_NAME_FULL, $tag_priority = '') {
+function makeTags(array $list, bool $html = true, string $key = 'eventid', int $list_tag_count = ZBX_TAG_COUNT_DEFAULT,
+		array $filter_tags = [], ?array $subfilter_tags = null, int $tag_name_format = TAG_NAME_FULL,
+		string $tag_priority = ''): array {
 	$tags = [];
 
 	if ($html) {
@@ -761,6 +553,14 @@ function makeTags(array $list, $html = true, $key = 'eventid', $list_tag_count =
 				$value = getTagString($tag, $tag_name_format);
 
 				if ($value !== '') {
+					if ($subfilter_tags !== null
+							&& !(array_key_exists($tag['tag'], $subfilter_tags)
+								&& array_key_exists($tag['value'], $subfilter_tags[$tag['tag']]))) {
+						$value = (new CLinkAction($value))->onClick(CHtml::encode(
+							'view.setSubfilter('.json_encode(['subfilter_tags['.$tag['tag'].'][]', $tag['value']]).')'
+						));
+					}
+
 					$tags[$element[$key]][] = (new CSpan($value))
 						->addClass(ZBX_STYLE_TAG)
 						->setHint(getTagString($tag));
@@ -780,16 +580,23 @@ function makeTags(array $list, $html = true, $key = 'eventid', $list_tag_count =
 
 				foreach ($element['tags'] as $tag) {
 					$value = getTagString($tag);
+
+					if ($subfilter_tags !== null
+							&& !(array_key_exists($tag['tag'], $subfilter_tags)
+								&& array_key_exists($tag['value'], $subfilter_tags[$tag['tag']]))) {
+						$value = (new CLinkAction($value))->onClick(CHtml::encode(
+							'view.setSubfilter('.json_encode(['subfilter_tags['.$tag['tag'].'][]', $tag['value']]).')'
+						));
+					}
+
 					$hint_content[$element[$key]][] = (new CSpan($value))
 						->addClass(ZBX_STYLE_TAG)
 						->setHint($value);
 				}
 
-				$tags[$element[$key]][] = (new CSpan(
-					(new CButton(null))
-						->addClass(ZBX_STYLE_ICON_WZRD_ACTION)
-						->setHint(new CDiv($hint_content), '', true, 'max-width: 500px')
-				))->addClass(ZBX_STYLE_REL_CONTAINER);
+				$tags[$element[$key]][] = (new CButton(null))
+					->addClass(ZBX_STYLE_ICON_WZRD_ACTION)
+					->setHint($hint_content, ZBX_STYLE_HINTBOX_WRAP, true);
 			}
 		}
 		else {
@@ -810,16 +617,16 @@ function makeTags(array $list, $html = true, $key = 'eventid', $list_tag_count =
  * @param array  $tag
  * @param string $tag['tag']
  * @param string $tag['value']
- * @param int    $tag_name_format  PROBLEMS_TAG_NAME_*
+ * @param int    $tag_name_format  TAG_NAME_*
  *
  * @return string
  */
-function getTagString(array $tag, $tag_name_format = PROBLEMS_TAG_NAME_FULL) {
+function getTagString(array $tag, $tag_name_format = TAG_NAME_FULL) {
 	switch ($tag_name_format) {
-		case PROBLEMS_TAG_NAME_NONE:
+		case TAG_NAME_NONE:
 			return $tag['value'];
 
-		case PROBLEMS_TAG_NAME_SHORTENED:
+		case TAG_NAME_SHORTENED:
 			return substr($tag['tag'], 0, 3).(($tag['value'] === '') ? '' : ': '.$tag['value']);
 
 		default:
