@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2021 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -23,6 +23,8 @@
  * @var CView $this
  */
 
+$this->includeJsFile('configuration.hostgroups.list.js.php');
+
 $widget = (new CWidget())
 	->setTitle(_('Host groups'))
 	->setControls((new CTag('nav', true, (new CList())
@@ -36,7 +38,8 @@ $widget = (new CWidget())
 			)
 		))->setAttribute('aria-label', _('Content controls'))
 	)
-	->addItem((new CFilter(new CUrl('hostgroups.php')))
+	->addItem((new CFilter())
+		->setResetUrl(new CUrl('hostgroups.php'))
 		->setProfile($data['profileIdx'])
 		->setActiveTab($data['active_tab'])
 		->addFilterTab(_('Filter'), [
@@ -119,10 +122,12 @@ foreach ($this->data['groups'] as $group) {
 		}
 
 		if ($data['allowed_ui_conf_hosts']) {
-			$host_output = (new CLink($host['name'], (new CUrl('hosts.php'))
-				->setArgument('form', 'update')
+			$host_output = (new CLink($host['name'], (new CUrl('zabbix.php'))
+				->setArgument('action', 'host.edit')
 				->setArgument('hostid', $host['hostid'])
-			))->addClass(ZBX_STYLE_LINK_ALT);
+			))
+				->onClick('view.editHost(event, '.json_encode($host['hostid']).')')
+				->addClass(ZBX_STYLE_LINK_ALT);
 		}
 		else {
 			$host_output = new CSpan($host['name']);
@@ -140,7 +145,9 @@ foreach ($this->data['groups'] as $group) {
 	if ($group['discoveryRule']) {
 		if ($data['allowed_ui_conf_hosts']) {
 			$lld_name = (new CLink($group['discoveryRule']['name'],
-				(new CUrl('host_prototypes.php'))->setArgument('parent_discoveryid', $group['discoveryRule']['itemid'])
+				(new CUrl('host_prototypes.php'))
+					->setArgument('parent_discoveryid', $group['discoveryRule']['itemid'])
+					->setArgument('context', 'host')
 			));
 		}
 		else {
@@ -163,7 +170,8 @@ foreach ($this->data['groups'] as $group) {
 		(new CCol($name))->addClass(ZBX_STYLE_NOWRAP),
 		[
 			$data['allowed_ui_conf_hosts']
-				? new CLink(_('Hosts'), (new CUrl('hosts.php'))
+				? new CLink(_('Hosts'), (new CUrl('zabbix.php'))
+					->setArgument('action', 'host.list')
 					->setArgument('filter_set', '1')
 					->setArgument('filter_groups', [$group['groupid']])
 				)
